package org.netkernel.rdbms.endpoint;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.rdbms.representation.IAspectDBConnectionPool;

import java.io.*;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Timer;
import java.util.TimerTask;

public abstract class AbstractRDBMSAccessor extends StandardAccessorImpl
{
	private static final String ARG_CONFIGURATION="configuration";
	private static final String DEFAULT_CONFIG="res:/etc/ConfigRDBMS.xml";
	private static final int PIPE_BUFFER_SIZE_BYTES=256;

	protected static final int INITIAL_CLOB_BUFFER_SIZE_BYTES=2048;

	private Timer mTimer;

	public String getConfigURI(INKFRequestContext context)
	{	String configURI;
		if (context.getThisRequest().argumentExists(ARG_CONFIGURATION))
		{	configURI="arg:"+ARG_CONFIGURATION;
		}
		else
		{	configURI=DEFAULT_CONFIG;
		}
		return configURI;
	}


	/**
	 * Will return a timer task registered with a timer to cancel the given statement if it doesn't
	 * complete within the timeout period specified in the connectionpool if this connection pool is
	 * configured to use the TIMER timeout method. Otherwise if using the SET method the
	 * statement will be configured with a query timeout will return null.
	 */
	public TimerTask configureStatementTimeout(Statement aStatement, IAspectDBConnectionPool aConnection) throws SQLException
	{	TimerTask tt=null;
		if (aConnection.getQueryTimeoutMethod()==IAspectDBConnectionPool.TIMEOUT_METHOD_TIMER)
		{	synchronized (this)
			{	if (mTimer==null)
				{	mTimer=new Timer();
				}
			}
			final Statement statement=aStatement;
			tt=new TimerTask()
			{	public void run()
				{	try
					{	statement.cancel();
					}
					catch (SQLException e)
					{ // nothing we can do
					}
				}
			};
			mTimer.schedule(tt, 1000L * (long) aConnection.getDefaultQueryTimeout());
		}
		else if (aConnection.getQueryTimeoutMethod()==IAspectDBConnectionPool.TIMEOUT_METHOD_SET)
		{	aStatement.setQueryTimeout(aConnection.getDefaultQueryTimeout());
		}
		return tt;
	}


	/**
	 * copy an reader to a writer and flush streams when finished
	 *
	 * @throws java.io.IOException if there are any problems
	 */
	protected static void pipe(Reader aInput, Writer aOutput) throws IOException
	{	char c[]=new char[PIPE_BUFFER_SIZE_BYTES];
		int n;
		try
		{	while ((n=aInput.read(c))>0)
			{	aOutput.write(c, 0, n);
			}
		}
		finally
		{	aOutput.flush();
		}
	}

	protected static void pipe(InputStream aInput, OutputStream aOutput) throws IOException
	{	byte b[]=new byte[PIPE_BUFFER_SIZE_BYTES];
		int c;
		try
		{	while ((c=aInput.read(b))>0)
			{	aOutput.write(b, 0, c);
			}
		}
		finally
		{	try
			{	aInput.close();
			}
			finally
			{	aOutput.close();
			}
		}
	}

}
