/******************************************************************************
 (c) Copyright 2002 - $Date: 2008/05/29 12:48:41 $ 1060 Research Ltd

 This Software is licensed to You, the licensee, for use under the terms of
 the 1060 Public License v1.0. Please read and agree to the 1060 Public
 License v1.0 [www.1060research.com/license] before using or redistributing
 this software.

 In summary the 1060 Public license has the following conditions.
 A. You may use the Software free of charge provided you agree to the terms
 laid out in the 1060 Public License v1.0
 B. You are only permitted to use the Software with components or applications
 that provide you with OSI Certified Open Source Code [www.opensource.org], or
 for which licensing has been approved by 1060 Research Limited.
 You may write your own software for execution by this Software provided any
 distribution of your software with this Software complies with terms set out
 in section 2 of the 1060 Public License v1.0
 C. You may redistribute the Software provided you comply with the terms of
 the 1060 Public License v1.0 and that no warranty is implied or given.
 D. If you find you are unable to comply with this license you may seek to
 obtain an alternative license from 1060 Research Limited by contacting
 license@1060research.com or by visiting www.1060research.com

 NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 LICENSE V1.0 FOR DETAILS

 THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license

 File:          $RCSfile: RDBMSUtil.java,v $
 Version:       $Name:  $ $Revision: 1.5 $
 Last Modified: $Date: 2008/05/29 12:48:41 $
 *****************************************************************************/


package org.netkernel.rdbms.util;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.util.RequestScopeClassLoader;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.request.IRequestScopeLevel;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;

/**
 * Utilities to convert from SQL ResultSets to HDS etc.
 *
 * @author pjr
 */
public class RDBMSUtil
{
	public static final String ROOT_NAME="resultset";
	public static final String ROW_NAME="row";

	/**
	 * Convert a Result Set to an HDS tree
	 */
	public static void resultsetToHDS(HDSBuilder aBuilder, ResultSet aResultSet) throws Exception
	{	aBuilder.pushNode(ROOT_NAME);
		ResultSetMetaData resultSetMetaData=aResultSet.getMetaData();
		int numberOfColumns=resultSetMetaData.getColumnCount();
		String[] columnNames=new String[numberOfColumns];
		for (int i=0; i<numberOfColumns; i++)
		{	columnNames[i]=resultSetMetaData.getColumnLabel(i+1);
		}
		while (aResultSet.next())
		{	processRow(aBuilder, aResultSet, columnNames);
		}
		aBuilder.popNode();
	}

	private static void processRow(HDSBuilder aBuilder, ResultSet aResultSet, String[] aColumnNames) throws Exception
	{	aBuilder.pushNode(ROW_NAME);
		for (int i=0; i<aColumnNames.length; i++)
		{	Object o=aResultSet.getObject(i+1);
			aBuilder.addNode(aColumnNames[i], o);
		}
		aBuilder.popNode();
	}

	public static Document resultsetToDOM(ResultSet aResultSet) throws Exception
	{	Document result=XMLUtils.newDocument();
		Element root=result.createElement(ROOT_NAME);
		result.appendChild(root);
		ResultSetMetaData resultSetMetaData=aResultSet.getMetaData();
		int numberOfColumns=resultSetMetaData.getColumnCount();
		String[] columnNames=new String[numberOfColumns];
		for (int i=0; i<numberOfColumns; i++)
		{	columnNames[i]=resultSetMetaData.getColumnLabel(i+1);
		}
		while (aResultSet.next())
		{	processRow(root, aResultSet, columnNames);
		}
		return result;
	}

	private static void processRow(Element aRoot, ResultSet aResultSet, String[] aColumnNames) throws Exception
	{	Document d=aRoot.getOwnerDocument();
		Element row=d.createElement(ROW_NAME);
		aRoot.appendChild(row);
		for (int i=0; i<aColumnNames.length; i++)
		{	Object o=aResultSet.getObject(i+1);
			String value=(o==null) ? "null" : o.toString();
			Element column=d.createElement(aColumnNames[i]);
			column.appendChild(d.createTextNode(value));
			row.appendChild(column);
		}
	}

	private static final char SINGLE_QUOTE='\'';
	private static final char BACKSLASH='\\';

	/**
	 * Escape database outbound data that may have been inescaped
	 */
	public static String outEscape(String s)
	{	char[] chars=s.toCharArray();
		int j=0;
		int i=0;
		int l=chars.length-1;
		for (i=0; i<l; i++)
		{	char c1=chars[i];
			char c2=chars[i+1];
			if ((c1==SINGLE_QUOTE && c2==SINGLE_QUOTE) || (c1==BACKSLASH && c2==BACKSLASH))
			{	i++;
			}
			chars[j++]=c1;
		}
		if (i==l)
		{	chars[j++]=chars[i];
		}
		return String.copyValueOf(chars, 0, j);
	}

	/**
	 * Escape database inbound data to be correct SQL
	 * For security reaplace all single quotes and backslashes ' --> '' and \ --> \\
	 */
	public static String inEscape(String s)
	{	char[] charsIn=s.toCharArray();
		int l=charsIn.length;
		char[] charsOut=new char[l * 2];
		int i=0;
		int j=0;
		for (i=0; i<l; i++)
		{	char c=charsIn[i];
			if (c==SINGLE_QUOTE || c==BACKSLASH)
			{	charsOut[j++]=c;
			}
			charsOut[j++]=c;
		}
		return String.copyValueOf(charsOut, 0, j);
	}
	
	public static void setContextClassloader(INKFRequestContext aContext) throws Exception
	{
		IRequestScopeLevel scope=aContext.getKernelContext().getRequestScope();
		RequestScopeClassLoader cl = new RequestScopeClassLoader(scope);
		Thread.currentThread().setContextClassLoader(cl);
	}
	
}
